@@ -149,11 +149,21 @@ function(model, ...)
     EN = double(c),
     ED = double(c),
     A = double(c * c),
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCombineComponentsMIX!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   model@tau <- matrix(data = output$tau, ncol = c, byrow = TRUE)
@@ -183,11 +193,21 @@ function(model, ...)
     theta2 = as.double(unlist(theta2)),
     theta3 = as.double(unlist(theta3)),
     Z = integer(n),
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCLRMIX!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   unique.Z <- unique(output$Z)
@@ -269,11 +289,21 @@ function(model, ...)
     EN = double(c),
     ED = double(c),
     A = double(c * c),    
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCombineComponentsMVNORM!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   model@tau <- matrix(data = output$tau, ncol = c, byrow = TRUE)
@@ -302,11 +332,21 @@ function(model, ...)
     theta1 = as.double(unlist(theta1)),
     theta2 = as.double(unlist(theta2)),
     Z = integer(n),
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCLRMVNORM!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   unique.Z <- unique(output$Z)
@@ -335,7 +375,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("RCLRMIX Version 2.15.0")
+  message("RCLRMIX Version 2.16.0")
 
   flush.console()
 
