@@ -58,11 +58,21 @@ function(model, ...)
     theta3 = as.double(unlist(theta3)),
     P = as.double(unlist(model@P)),
     Z = integer(model@ntest),
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCLSMIX!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   model@Zp <- factor(output$Z, levels = levels(model@Zt))
@@ -125,11 +135,21 @@ function(model, ...)
     theta2 = as.double(unlist(theta2)),
     P = as.double(unlist(model@P)),
     Z = integer(model@ntest),
-    error = integer(1),
+    error = integer(9),
     PACKAGE = "rebmix")
 
-  if (output$error == 1) {
-    stop("in RCLSMVNORM!", call. = FALSE); return(NA)
+  error <- error.to.string(output$error);
+      
+  if (error[1] != "") {
+    stop(error[1], call. = FALSE); return(NA)
+  }
+    
+  if (error[2] != "") {
+    warning(error[2], call. = FALSE, immediate. = TRUE)
+  }  
+    
+  if (error[3] != "") {
+    warning(error[3], call. = FALSE, immediate. = TRUE)
   }
 
   model@Zp <- factor(output$Z, levels = levels(model@Zt))
@@ -148,7 +168,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("RCLSMIX Version 2.15.0")
+  message("RCLSMIX Version 2.16.0")
 
   flush.console()
 
@@ -194,7 +214,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("BFSMIX Version 2.15.0")
+  message("BFSMIX Version 2.16.0")
 
   flush.console()
 
