@@ -117,6 +117,7 @@ function(model, ...)
       h = as.double(h),      
       ar = as.double(model@ar),
       Restraints = as.character(model@Restraints),
+      Mode = as.character(model@Mode),
       n = as.integer(n),
       Y = as.double(X),
       Y.type = as.integer(Y.type),
@@ -148,16 +149,27 @@ function(model, ...)
       opt.c = integer(1000), # 1000 = ItMax see rebmixf.h
       opt.IC = double(1000),
       opt.logL = double(1000),
+      opt.Dmin = double(1000),
       opt.D = double(1000),
       all.length = integer(1),
       all.K = integer(dK),
       all.IC = double(dK),
-      error = integer(1),
+      error = integer(9),
       PACKAGE = "rebmix")
-
-    if (output$error == 1) {
-      stop("in RREBMIX!", call. = FALSE); return(NA) #
+      
+    error <- error.to.string(output$error);
+      
+    if (error[1] != "") {
+      stop(error[1], call. = FALSE); return(NA)
     }
+    
+    if (error[2] != "") {
+      warning(error[2], call. = FALSE, immediate. = TRUE)
+    }  
+    
+    if (error[3] != "") {
+      warning(error[3], call. = FALSE, immediate. = TRUE)
+    }  
 
     c <- output$summary.c
 
@@ -170,6 +182,7 @@ function(model, ...)
     length(output$opt.c) <- output$opt.length
     length(output$opt.IC) <- output$opt.length
     length(output$opt.logL) <- output$opt.length
+    length(output$opt.Dmin) <- output$opt.length
     length(output$opt.D) <- output$opt.length
 
     j <- order(output$opt.c, output$opt.logL)
@@ -177,6 +190,7 @@ function(model, ...)
     output$opt.c <- output$opt.c[j]
     output$opt.IC <- output$opt.IC[j]
     output$opt.logL <- output$opt.logL[j]
+    output$opt.Dmin <- output$opt.Dmin[j]
     output$opt.D <- output$opt.D[j]
 
     j <- !duplicated(output$opt.c, fromLast = TRUE)
@@ -184,6 +198,7 @@ function(model, ...)
     output$opt.c <- output$opt.c[j]
     output$opt.IC <- output$opt.IC[j]
     output$opt.logL <- output$opt.logL[j]
+    output$opt.Dmin <- output$opt.Dmin[j]
     output$opt.D <- output$opt.D[j]
 
     length(output$all.K) <- output$all.length
@@ -226,6 +241,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -246,6 +262,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -265,6 +282,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -286,6 +304,7 @@ function(model, ...)
         output$Criterion,
         output$ar,
         output$Restraints,
+        output$Mode,
         output$summary.c,
         NA,
         NA,
@@ -312,6 +331,7 @@ function(model, ...)
     model@opt.c[[i]] <- output$opt.c
     model@opt.IC[[i]] <- output$opt.IC
     model@opt.logL[[i]] <- output$opt.logL
+    model@opt.Dmin[[i]] <- output$opt.Dmin
     model@opt.D[[i]] <- output$opt.D
     model@all.K[[i]] <- output$all.K
     model@all.IC[[i]] <- output$all.IC
@@ -330,6 +350,7 @@ function(model, ...)
     "Criterion",
     "ar",
     "Restraints",
+    "Mode",
     "c",
     "v/k",
     "K",
@@ -470,6 +491,7 @@ function(model, ...)
       h = as.double(h),       
       ar = as.double(model@ar),
       Restraints = as.character(model@Restraints),
+      Mode = as.character(model@Mode),
       n = as.integer(n),
       Y = as.double(X),
       Y.type = as.integer(Y.type),
@@ -500,16 +522,27 @@ function(model, ...)
       opt.c = integer(1000), # 1000 = ItMax see rebmixf.h
       opt.IC = double(1000),
       opt.logL = double(1000),
+      opt.Dmin = double(1000),
       opt.D = double(1000),
       all.length = integer(1),
       all.K = integer(dK),
       all.IC = double(dK),
-      error = integer(1),
+      error = integer(9),
       PACKAGE = "rebmix")
 
-    if (output$error == 1) {
-      stop("in RREBMVNORM!", call. = FALSE); return(NA)
+    error <- error.to.string(output$error);
+      
+    if (error[1] != "") {
+      stop(error[1], call. = FALSE); return(NA)
     }
+    
+    if (error[2] != "") {
+      warning(error[2], call. = FALSE, immediate. = TRUE)
+    }  
+    
+    if (error[3] != "") {
+      warning(error[3], call. = FALSE, immediate. = TRUE)
+    } 
 
     c <- output$summary.c
 
@@ -521,6 +554,7 @@ function(model, ...)
     length(output$opt.c) <- output$opt.length
     length(output$opt.IC) <- output$opt.length
     length(output$opt.logL) <- output$opt.length
+    length(output$opt.Dmin) <- output$opt.length
     length(output$opt.D) <- output$opt.length
 
     j <- order(output$opt.c, output$opt.logL)
@@ -528,6 +562,7 @@ function(model, ...)
     output$opt.c <- output$opt.c[j]
     output$opt.IC <- output$opt.IC[j]
     output$opt.logL <- output$opt.logL[j]
+    output$opt.Dmin <- output$opt.Dmin[j]
     output$opt.D <- output$opt.D[j]
 
     j <- !duplicated(output$opt.c, fromLast = TRUE)
@@ -535,6 +570,7 @@ function(model, ...)
     output$opt.c <- output$opt.c[j]
     output$opt.IC <- output$opt.IC[j]
     output$opt.logL <- output$opt.logL[j]
+    output$opt.Dmin <- output$opt.Dmin[j]
     output$opt.D <- output$opt.D[j]
 
     length(output$all.K) <- output$all.length
@@ -573,6 +609,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -593,6 +630,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -612,6 +650,7 @@ function(model, ...)
           output$Criterion,
           output$ar,
           output$Restraints,
+          output$Mode,
           output$summary.c,
           output$summary.k,
           output$K,
@@ -633,6 +672,7 @@ function(model, ...)
         output$Criterion,
         output$ar,
         output$Restraints,
+        output$Mode,
         output$summary.c,
         NA,
         NA,
@@ -660,6 +700,7 @@ function(model, ...)
     model@opt.c[[i]] <- output$opt.c
     model@opt.IC[[i]] <- output$opt.IC
     model@opt.logL[[i]] <- output$opt.logL
+    model@opt.Dmin[[i]] <- output$opt.Dmin
     model@opt.D[[i]] <- output$opt.D
     model@all.K[[i]] <- output$all.K
     model@all.IC[[i]] <- output$all.IC
@@ -678,6 +719,7 @@ function(model, ...)
     "Criterion",
     "ar",
     "Restraints",
+    "Mode",
     "c",
     "v/k",
     "K",
@@ -723,13 +765,14 @@ function(model,
   ymax,
   ar,
   Restraints,
+  Mode,
 ### Panic Branislav.
   EMcontrol, ...)
 ### End    
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("REBMIX Version 2.15.0")
+  message("REBMIX Version 2.16.0")
 
   flush.console()
 
@@ -748,6 +791,7 @@ function(model,
      ymax = ymax,
      ar = ar,
      Restraints = Restraints,
+     Mode = Mode,
 ### Panic Branislav.
      EMcontrol = EMcontrol)
 ### End     
@@ -779,6 +823,7 @@ function(model,
     model@opt.c[[length(model@opt.c) + 1]] <- output@opt.c[[k]]
     model@opt.IC[[length(model@opt.IC) + 1]] <- output@opt.IC[[k]]
     model@opt.logL[[length(model@opt.logL) + 1]] <- output@opt.logL[[k]]
+    model@opt.Dmin[[length(model@opt.Dmin) + 1]] <- output@opt.Dmin[[k]]
     model@opt.D[[length(model@opt.D) + 1]] <- output@opt.D[[k]]
     model@all.K[[length(model@all.K) + 1]] <- output@all.K[[k]]
     model@all.IC[[length(model@all.IC) + 1]] <- output@all.IC[[k]]
