@@ -45,12 +45,22 @@ function(model, ...)
       n = integer(1),
       Y = double(sum(model@n) * d),
       Z = integer(sum(model@n)),
-      error = integer(1),
+      error = integer(9),
       PACKAGE = "rebmix")
 
-    if (output$error == 1) {
-      stop("in RNGMIX!", call. = FALSE); return(NA)
+    error <- error.to.string(output$error);
+      
+    if (error[1] != "") {
+      stop(error[1], call. = FALSE); return(NA)
     }
+    
+    if (error[2] != "") {
+      warning(error[2], call. = FALSE, immediate. = TRUE)
+    }  
+    
+    if (error[3] != "") {
+      warning(error[3], call. = FALSE, immediate. = TRUE)
+    } 
 
     dim(output$Y) <- c(output$n, d)
 
@@ -122,12 +132,22 @@ function(model, ...)
       n = integer(1),
       Y = double(sum(model@n) * d),
       Z = integer(sum(model@n)),
-      error = integer(1),
+      error = integer(9),
       PACKAGE = "rebmix")
 
-    if (output$error == 1) {
-      stop("in RRNGMVNORM!", call. = FALSE); return(NA)
+    error <- error.to.string(output$error);
+      
+    if (error[1] != "") {
+      stop(error[1], call. = FALSE); return(NA)
     }
+    
+    if (error[2] != "") {
+      warning(error[2], call. = FALSE, immediate. = TRUE)
+    }  
+    
+    if (error[3] != "") {
+      warning(error[3], call. = FALSE, immediate. = TRUE)
+    } 
 
     dim(output$Y) <- c(output$n, d)
 
@@ -167,7 +187,7 @@ function(model,
 {
   digits <- getOption("digits"); options(digits = 15)
 
-  message("RNGMIX Version 2.15.0")
+  message("RNGMIX Version 2.16.0")
 
   flush.console()
 
