@@ -6,12 +6,13 @@ pdf = c("normal", "lognormal", "Weibull", "binomial", "Poisson", "Dirac", "gamma
 pdf.nargs = c(2, 2, 2, 2, 1, 1, 2, 2, 2, 3),
 pdf.Variables = c("continuous", "continuous", "continuous", "discrete", "discrete", "discrete", "continuous", "continuous", "continuous", "continuous"),
 Restraints = c("rigid", "loose"),
+Mode = c("all", "outliers", "outliersplus"),
 ### Panic Branislav.
 EMStrategy = c("none", "exhaustive", "best", "single"),
 EMVariant = c("EM", "ECM"),
 EMAcceleration = c("fixed", "line", "golden")),
 ### End
-.Names = c("Preprocessing", "Criterion", "Variables", "pdf", "pdf.nargs", "pdf.Variables", "Restraints", "EMStrategy", "EMVariant", "EMAcceleration"))
+.Names = c("Preprocessing", "Criterion", "Variables", "pdf", "pdf.nargs", "pdf.Variables", "Restraints", "Mode", "EMStrategy", "EMVariant", "EMAcceleration"))
 
 .rebmix.plot <- structure(list(
 what = c("pdf", "marginal pdf", "IC", "logL", "D", "marginal cdf", "K", "cdf")),
@@ -29,6 +30,12 @@ Rule = c("Sturges", "Log10", "RootN", "Knuth equal", "Knuth unequal")),
 Rule = c("Entropy", "Demp")),
 .Names = c("Rule"))
 
+.error.defaults <- structure(list(
+ErrorNames = c("E_OK", "E_MEM", "E_ARG", "E_CON", "E_FILE", "E_NO_SOLUTION"),
+FileNames = c("base.cpp", "rngmixf.cpp", "rngmvnormf.cpp", "rebmixf.cpp", "rebmvnormf.cpp", "emf.cpp", "Rmisc.cpp", "Rrebmix.cpp", "Rrebmvnorm.cpp")),
+.Names = c("ErrorNames", "FileNames"))
+
+
 
 
 
