@@ -44,7 +44,22 @@
         Mean = as.double(xTheta[[i]]$theta1),
         Kappa = as.double(xTheta[[i]]$theta2),
         F = double(n),
+        error = integer(9),
         PACKAGE = "rebmix")
+        
+      error <- error.to.string(output$error);
+      
+      if (error[1] != "") {
+        stop(error[1], call. = FALSE); return(NA)
+      }
+   
+      if (error[2] != "") {
+        warning(error[2], call. = FALSE, immediate. = TRUE)
+      }  
+    
+      if (error[3] != "") {
+        warning(error[3], call. = FALSE, immediate. = TRUE)
+      }        
 
       fix <- output$F
     }
@@ -101,7 +116,22 @@
         Mean = as.double(yTheta[[i]]$theta1),
         Kappa = as.double(yTheta[[i]]$theta2),
         F = double(n),
+        error = integer(9),
         PACKAGE = "rebmix")
+        
+      error <- error.to.string(output$error);
+      
+      if (error[1] != "") {
+        stop(error[1], call. = FALSE); return(NA)
+      }
+   
+      if (error[2] != "") {
+        warning(error[2], call. = FALSE, immediate. = TRUE)
+      }  
+    
+      if (error[3] != "") {
+        warning(error[3], call. = FALSE, immediate. = TRUE)
+      }       
 
       fiy <- output$F
     }
