@@ -3,7 +3,7 @@
   f <- array(data = 0.0, dim = length(x), dimnames = NULL)
 
   for (i in 1:length(w)) {
-    fi <- pnorm(as.numeric(x), mean = as.numeric(Theta[[i]]$theta1[xi]), sd = as.numeric(Theta[[i]]$theta2[xi]^0.5), ...)
+    fi <- pnorm(as.numeric(x), mean = as.numeric(Theta[[i]]$theta1[xi]), sd = as.numeric(Theta[[i]]$theta2[xi, xi]^0.5), ...)
 
     f <- f + w[i] * fi
   }
