@@ -6,7 +6,7 @@ extern void RRNGMIX(void *, void *, void *, void *, void *, void *, void *, void
 extern void RREBMIX(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
     void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
     void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
-    void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
+    void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
 extern void RdensKNearestNeighbourXY(void *, void *, void *, void *, void *, void *, void *, void *);
 extern void RdensKDEXY(void *, void *, void *, void *, void *, void *, void *);
@@ -45,7 +45,7 @@ extern void RRNGMVNORM(void *, void *, void *, void *, void *, void *, void *, v
 extern void RREBMVNORM(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
     void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
     void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *,
-    void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
+    void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
 extern void RCLSMVNORM(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
@@ -69,7 +69,7 @@ extern void REMMVNORM(void *, void *, void *, void *, void *, void *, void *, vo
 /// End
 
 extern void RvonMisesPdf(void *, void *, void *, void *, void *);
-extern void RvonMisesCdf(void *, void *, void *, void *, void *);
+extern void RvonMisesCdf(void *, void *, void *, void *, void *, void *);
 
 extern void RGumbelPdf(void *, void *, void *, void *, void *, void *);
 extern void RGumbelCdf(void *, void *, void *, void *, void *, void *);
@@ -78,7 +78,7 @@ extern void Roptbins(void *, void *, void *, void *, void *, void *, void *, voi
 extern void Rbins(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
 extern void RTvtNormalPdf(void *, void *, void *, void *, void *, void *);
-extern void RMvtNormalPdf(void *, void *, void *, void *, void *, void *);
+extern void RMvtNormalPdf(void *, void *, void *, void *, void *, void *, void *);
 
 extern void Rfhistogram(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void Rchistogram(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
@@ -89,7 +89,7 @@ extern void RMergeLabels(void *, void *, void *, void *, void *, void *);
 
 static const R_CMethodDef CMethods[] = {
     {"RRNGMIX", (DL_FUNC) &RRNGMIX, 13},
-    {"RREBMIX", (DL_FUNC) &RREBMIX, 55},
+    {"RREBMIX", (DL_FUNC) &RREBMIX, 57},
     {"RdensKNearestNeighbourXY", (DL_FUNC) &RdensKNearestNeighbourXY, 8},
     {"RdensKDEXY", (DL_FUNC) &RdensKDEXY, 7},
     {"RdensHistogramXY", (DL_FUNC) &RdensHistogramXY, 16},
@@ -110,7 +110,7 @@ static const R_CMethodDef CMethods[] = {
     {"RInformationCriterionMIX", (DL_FUNC)&RInformationCriterionMIX, 18},
     {"RCombineComponentsMIX", (DL_FUNC) &RCombineComponentsMIX, 18},
     {"RRNGMVNORM", (DL_FUNC) &RRNGMVNORM, 12},
-    {"RREBMVNORM", (DL_FUNC) &RREBMVNORM, 54},
+    {"RREBMVNORM", (DL_FUNC) &RREBMVNORM, 56},
     {"RCLSMVNORM", (DL_FUNC) &RCLSMVNORM, 13},
     {"RCLRMVNORM", (DL_FUNC) &RCLRMVNORM, 10},
     {"RPreprocessingKNNMVNORM", (DL_FUNC) &RPreprocessingKNNMVNORM, 7},
@@ -124,13 +124,13 @@ static const R_CMethodDef CMethods[] = {
     {"RInformationCriterionMVNORM", (DL_FUNC)&RInformationCriterionMVNORM, 15},
     {"RCombineComponentsMVNORM", (DL_FUNC) &RCombineComponentsMVNORM, 18},
     {"RvonMisesPdf", (DL_FUNC) &RvonMisesPdf, 5},
-    {"RvonMisesCdf", (DL_FUNC) &RvonMisesCdf, 5},
+    {"RvonMisesCdf", (DL_FUNC) &RvonMisesCdf, 6},
     {"RGumbelPdf", (DL_FUNC) &RGumbelPdf, 6},
     {"RGumbelCdf", (DL_FUNC) &RGumbelCdf, 6},
     {"Roptbins", (DL_FUNC) &Roptbins, 12},
     {"Rbins", (DL_FUNC) &Rbins, 11},
     {"RTvtNormalPdf", (DL_FUNC) &RTvtNormalPdf, 6},
-    {"RMvtNormalPdf", (DL_FUNC) &RMvtNormalPdf, 6},
+    {"RMvtNormalPdf", (DL_FUNC) &RMvtNormalPdf, 7},
 /// Panic Branislav
     {"REMMIX", (DL_FUNC)&REMMIX, 21},
     {"REMMVNORM", (DL_FUNC)&REMMVNORM, 20},
